import {
  csv,
  text,
  tsv,
  select,
  forceSimulation,
  forceLink,
  forceManyBody,
  forceCenter
} from 'd3';

const drag = simulation => {
  function dragstarted(event) {
    if (!event.active) simulation.alphaTarget(0.3).restart();
    event.subject.fx = event.subject.x;
    event.subject.fy = event.subject.y;
  }
  
  function dragged(event) {
    event.subject.fx = event.x;
    event.subject.fy = event.y;
  }
  
  function dragended(event) {
    if (!event.active) simulation.alphaTarget(0);
    event.subject.fx = null;
    event.subject.fy = null;
  }
  
  
  return d3.drag()
      .on("start", dragstarted)
      .on("drag", dragged)
      .on("end", dragended);
}

var highlightLink = [];
var allLinkID = {};

const renderAD = (matrix, nodes) => {
  const margin = { top: 40, right: 40, bottom: 50, left: 100};
	const size = 15;
  var svg2=d3.select("#ad").append("svg")
  .attr("width", 9000)
  .attr("height", 9000)
  .append("g")
  
  svg2.append('text')
      .attr('id', 't1')
      .attr('y', +35)
      .attr('x', +50)
      .attr("fill", "#656565")
  	  .attr("font-size",25)
      .text('Adjacency Matrix');
  
  matrix.forEach((m,i) => {
    svg2.append("g")
      .attr("transform","translate(50,50)")
      .attr("id","row"+(i+1))
      .selectAll("rect")
      .data(m)
      .enter()
      .append("rect")
    	.style("stroke", "#bfbfbf")
    	.style("fill", "#656565")
      .attr("width",size)
      .attr("height",size)
      .attr("x", d=> d.x*size)
      .attr("y", d=> d.y*size)
    	.attr("id", d=> "g" + (d.x+1)+"x"+(d.y+1))
    	
      .style("fill-opacity", d=> d.weight * 1)

    	.on("click", datum => {
      	d3.selectAll("rect").style("stroke-width", "1px").style("stroke", "#bfbfbf");
      	
      	d3.select("#"+datum.path[0].id).style("stroke-width", "3px").style("stroke", "#965454");
      	if(datum.path[0].style['fillOpacity'] == "0"){
          var tmp = datum.path[0].id.replace('g', '');
          tmp = tmp.split('x');
        	d3.select("#t1").text("There is no link between node " + tmp[0] + " and node " + tmp[1]);
        }else{
          var tmp = datum.path[0].id.replace('g', '');
          tmp = tmp.split('x');
         
        		d3.select("#t1")
            	.attr("fill", "#656565")
  	  				.attr("font-size",25)
              .text("The link between node " + tmp[0] + " and node " + tmp[1] + " has been highlighted.");
        		
        		
        }
      })
  });  

  const classColor = {"1": "#69647b",
                      "2": "#e2cdbc",
                     	"3": "#a56c41",
                      "4": "#2d241f",
                      "5": "#7b8b6f"};
  
	svg2
		.append("g")
		.attr("transform","translate(50,47)")
		.selectAll("text")
		.data(nodes)
		.enter()
		.append("text")
		.attr("x", (d,i) => i * size + size/2)
		.text(d => d.id)
		.style("text-anchor","middle")
		.style("font-size","5px")
  	.style("fill", function(d) { return classColor[d.group];})

	svg2
		.append("g").attr("transform","translate(45,52.5)")
		.selectAll("text")
		.data(nodes)
		.enter()
		.append("text")
		.attr("y",(d,i) => i * size + size/2)
		.text(d => d.id)
		.style("text-anchor","middle")
		.style("font-size","5px")
		.style("fill", function(d) { return classColor[d.group];})
};

// Read Dataset
d3.csv('infect-dublin.edges').then( data => {
  var max = 0;
  var min = 9999;
  var links = [];
  var countLinks = [];
  for (var i = 0; i <= 1000; i++) {
  	countLinks.push(0);
  }
  var key = Object.keys(data[0])[0];
  links.push({source: key.split(' ')[0], target: key.split(' ')[1], value : 1});
  allLinkID["L"+key.split(' ')[0]+"x"+key.split(' ')[1]] = 1;
  data.forEach((d) => {
    key = Object.keys(d);
    var tmp = d[Object.keys(d)].split(' ');
    delete d[Object.keys(d)];
    d.source = tmp[0];
    d.target = tmp[1];
    d.value = 1;
    if(parseInt(tmp[0], 10) > max) max = d.source;
    if(parseInt(tmp[1], 10) > max) max = d.target;
    if(parseInt(tmp[0], 10) < min) min = d.source;
    if(parseInt(tmp[1], 10) < min) min = d.target;
    links.push(d);
    allLinkID["L"+d.source+"x"+d.target] = 1;
    countLinks[parseInt(tmp[0], 10)] += 1;
    countLinks[parseInt(tmp[1], 10)] += 1;
  });
  var nodes = [];
	for (var i = min; i <= max; i++) {
    nodes.push({
    id: String(i),
    group: Math.ceil(countLinks[i]/10)
			});
	}
	var edgeHash = {};
  links.forEach((d) => {
    var id = d.source + "-" + d.target;
    edgeHash[id] = 1;
    var id = d.target + "-" + d.source;
    edgeHash[id] = 1;
  });
  
  var matrix = [];
  for(var y=min; y<=max; y++) {
    var row = [];
  	for(var x=min; x<=max; x++) {
    	var grid = {x: x-1, y: y-1, weight: 0};
      if(edgeHash[y + "-" + x]){
      	grid.weight = 1;
      }
      row.push(grid);
    }
    matrix.push(row);
  }
  function optimal_leaf_order_permute() {
  var transpose = reorder.transpose(matrix),
      dist_rows = reorder.dist()(matrix),
      dist_cols = reorder.dist()(transpose),
      order = reorder.optimal_leaf_order(),
      row_perm = order.distanceMatrix(dist_rows)(matrix),
      col_perm = order.distanceMatrix(dist_cols)(transpose);
      
      matrix.push(row_perm);
}
  console.log(matrix);
  console.log(nodes);
  renderAD(matrix, nodes);
});